/** @type {import('tailwindcss').Config} */
export default {
  content: [
    "./index.html",
    "./src/**/*.{js,ts,jsx,tsx}",
  ],
  darkMode: ['class', ':root:not(.light)'],
  theme: {
    extend: {},
  },
  plugins: [],
}
